$title  Check benchmark data.
display "com: Check benchmark data.";
$ontext
Copyright (C)   2003 Shiro Takeda
Time-stamp: 	<2014-03-24 11:21:38 st>

$Id: check_benchmark_data.gms,v 1.1 2005/08/04 08:21:39 st Exp $
version:
Author: 	Shiro Takeda <zbc08106@park.zero.ad.jp>
e-mail:		<zbc08106@park.zero.ad.jp>
First-written:	<2003/04/24>
$offtext

*	----------------------------------------------------------------------
*	Important parameters.

*	The number of periods.
$if not setglobal tyear $setglobal tyear 30

*	Reduction rate (%).
$if not setglobal rdrate $setglobal rdrate 100

*	Suffix for plotting
$if not setglobal suffix $setglobal suffix %rdrate%

*	Balance of payments restriction.
$if not setglobal bopcon $setglobal bopcon 0

*	Quadratic adjustment cost.
$if not setglobal phi $setglobal phi 0.5

*	Solver
$if not setglobal mcp $setglobal mcp 1

*	Period for plotting.
$if not setglobal syear $setglobal syear 10

parameter
	sol_mcp	Flag for quadratic programming
	rdrate	Reduction rate;
sol_mcp = %mcp%;
rdrate = %rdrate%;
display sol_mcp, rdrate;

*	----------------------------------------------------------------------
*	Define elasticity parameters.
display "com: Define elasticity parameters";

*	NB: EOS means elasticity of substitution, EOT means elasticity of
*	transformation.
parameter
	eta	EOT: export and domestic supply			/ 4 /
	sigpfe	EOS: Primary factor and energy			/ 0.5 /
	sigpf	EOS: Primary factor
	sigee	EOS: energy input				/ 0.5 /

 	siga	Armington elasticity

 	sigu	EOS: Intertemporal elasticity			/ 0.5 /
 	sigcl	EOS: consumption and leisure
	sigcc	EOS: energy vs non-energy consumption		/ 0.3 /
	sigc	EOS: non-energy consumption goods		/ 1 /
	sigec	EOS: energy consumption goods			/ 2 /

  	eols	Elasticity of labor supply			/ 0.15 /
;
display eta, sigpfe, sigee, sigu, sigcc, sigc, sigec;

$ontext
The values of Armington elasticity (elasticity of substitution between
domestic good and import) and elasticity of substitution between primary
factors are taken from GTAP version 5 data.

$offtext
display "com: Defining sets and parameters for dynamic model.";

set
	t		Period index	/ 1*%tyear% /
	tfirst(t)	First period
	tlast(t)	Last period

 	tsub(t)	/ 1*%syear% /
*  	tdecade(t)	/ 1,11,21 /
* 	te	/ 1*%lyear% /
* 	tef(te);
;

tfirst(t) = yes$(ord(t) eq 1);
tlast(t) =  yes$(ord(t) eq card(t));

display t, tfirst, tlast;
alias (t,tt);

* display tsub;

* tef(te) = yes$(ord(te) > card(t));
* display te, tef;

*	Solver choice
option mcp = path;
* option mcp = miles;

*	End-of-line comment symbol.
$eolcom !

*	Inline comment symbol.
$inlinecom { }

*	Digit.
option decimals = 8;

option sysout = on;

*	----------------------------------------------------------------------
*	Import set definitions.
display "com: Import set definitions.";

$include set_definition

*	----------------------------------------------------------------------
*	Import benchmark data.
display "com: Import benchmark data.";

$include dataset.dat

*	Endogenous sector.
display "Endogenous sector";
display iov, iovc, iovnc, ioqc, ioqnc;

*	Value added sector.
display "Value added sector";
display nlabinc, ncapinc, labtax, captax, nitax;

*	Final demand sector.
display "Final demand sector";
display cons, gexp, inve, expo, impo, impt, ctax, csub;

*	Household data.
display "Household data";
display lincome, cincome, linctax, cinctax, hctax, hcsub, hsave, hntrans;

*	----------------------------------------------------------------------
*	Check imported data.
display "com: Check imported data.";

parameter ioq;
ioq(e,i) = ioqc(e,i) + ioqnc(e,i);
display ioq;

parameter
	chk_fdva	    Check FD and VA data
	chk_out	    Check output data
	chk_out_p	    -1 if no output

*	Parameters for aggregation.
	temp_iov,
	temp_ioqc,
	temp_ioqnc

	temp_nlabinc,
	temp_labtax,
	temp_ncapinc,
	temp_captax,
	temp_nitax,

	temp_cons,
	temp_gcon,
	temp_gcol,
	temp_inve,
	temp_expo,
	temp_impo,
	temp_impt;

*	----------------------------------------------------------------------
*	Check imported data.
display "com: Check imported data.";

parameter
	chk_fdva	Check FD and VA data
	chk_out		Check output data
	chk_out_p	-1 if no output;

*	Final demand and value added.
chk_fdva(i,"va") = nlabinc(i) + labtax(i) + ncapinc(i) + captax(i)
    + nitax(i);
chk_fdva("sum","va") = round(sum(i, chk_fdva(i,"va")), 6);
chk_fdva(i,"fd")
    = cons(i) + csub(i) + gexp(i) + inve(i)
    + expo(i) + impo(i) + impt(i);
chk_fdva("sum","fd") = round(sum(i, chk_fdva(i,"fd")), 6);

chk_fdva("diff","fd-va")
    = round(chk_fdva("sum","fd")
    - chk_fdva("sum","va"), 6);
display chk_fdva;
abort$chk_fdva("diff","fd-va") "FD and VA are inconsistent.";

*	Ouput.
chk_out(i,"sec")
    = sum(ii, iov(ii,i))
    + nlabinc(i) + labtax(i) + ncapinc(i) + captax(i)
    + nitax(i);
chk_out(i,"com")
    = sum(ii, iov(i,ii))
    + cons(i) + csub(i) + gexp(i) + inve(i)
    + expo(i) + impo(i) + impt(i);
chk_out("sum","sec") = sum(i, chk_out(i,"sec"));
chk_out("sum","com") = sum(i, chk_out(i,"com"));
chk_out(i,"sec_com") = round(chk_out(i,"sec") - chk_out(i,"com"), 6);
chk_out("sum","sec_com")
    = round(chk_out("sum","sec")
    - chk_out("sum","com"), 6);
display chk_out;

abort$chk_out("sum","sec_com") "Output data are inconsistent.";

chk_out(i,"sec") = round(chk_out(i,"sec"), 6);
chk_out(i,"com") = round(chk_out(i,"com"), 6);
chk_out_p(i,"sec") = -1;
chk_out_p(i,"com") = -1;
chk_out_p(i,"sec")$chk_out(i,"sec") = 1;
chk_out_p(i,"com")$chk_out(i,"com") = 1;
option chk_out_p:0;
display chk_out_p;

$ontext
The values of Armington elasticity (elasticity of substitution between
domestic good and import) and elasticity of substitution between value added
are taken from GTAP version 5 data.

$offtext
siga("agr") = 2.2;
siga("omi") = 2.8;
siga("lim") = 2.8;
siga("coc") = 2.8;
siga("sla") = 2.8;
siga("cru") = 2.8;
siga("nat") = 2.8;
siga("foo") = 2.2;
siga("tet") = 2.2;
siga("ppp") = 1.8;
siga("chm") = 1.9;
siga("pet") = 1.9;
siga("opp") = 1.9;
siga("cok") = 1.9;
siga("csc") = 1.9;
siga("iam") = 2.8;
siga("mac") = 2.8;
siga("oip") = 2.8;
siga("con") = 1.9;
siga("ele") = 2.8;
siga("gas") = 2.8;
siga("sww") = 2.8;
siga("com") = 1.9;
siga("res") = 1.9;
siga("tcb") = 1.9;
siga("pub") = 1.9;
siga("ser") = 1.9;
display siga;

sigpf("agr") = 0.237;
sigpf("omi") = 0.2;
sigpf("lim") = 0.2;
sigpf("coc") = 0.2;
sigpf("sla") = 0.2;
sigpf("cru") = 0.2;
sigpf("nat") = 0.2;
sigpf("foo") = 0.237;
sigpf("tet") = 1.26;
sigpf("ppp") = 1.26;
sigpf("chm") = 1.26;
sigpf("pet") = 1.26;
sigpf("opp") = 1.26;
sigpf("cok") = 1.26;
sigpf("csc") = 1.26;
sigpf("iam") = 1.26;
sigpf("mac") = 1.26;
sigpf("oip") = 1.26;
sigpf("con") = 1.4;
sigpf("ele") = 1.26;
sigpf("gas") = 1.26;
sigpf("sww") = 1.26;
sigpf("com") = 1.68;
sigpf("res") = 1.26;
sigpf("tcb") = 1.26;
sigpf("pub") = 1.26;
sigpf("ser") = 1.26;
display sigpf;

*	----------------------------------------------------------------------
*	Production side
display "com: Production side";

parameter
	int0(i,ii)	Intermediate inputs
	vld0(i)		Value of labor inputs (net of tax)
	vkd0(i)		Value of capital inputs (net of tax)
	vva0(i)		Gross value added
	ve0(i)		Value of energy inputs
	vvae0(i)	Value of VA and energy inputs

	outs0(i)	Sector output (gross of tax)
	outc0(i)	Commodity output (gross of tax)

	d0(i)		Value of domestic output
	x0(i)		Value of export supply
	out0(i)		Value of output (gross of tax))

	tq0(i)		Net indirect tax rate
	tl0(i)		Labor tax rate on production
	tk0(i)		Capital tax rate on production
;

*	Intermediate inputs.
int0(i,ii) = iov(i,ii);

*	Factor input (value).
vld0(i) = nlabinc(i);
vkd0(i) = ncapinc(i);

*	Sector output.
outs0(i) = sum(ii, iov(ii,i))
    + nlabinc(i) + ncapinc(i)
    + labtax(i) + captax(i) + nitax(i);
outs0(i) = outs0(i);

*	Commodity output.
outc0(i) = sum(ii, iov(i,ii))
    + cons(i) + csub(i) + gexp(i)
    + inve(i) + expo(i) + impo(i) + impt(i);
outc0(i) = outc0(i);

*	Output to export market.
x0(i) = expo(i);

*	Output to domestic market.
d0(i) = round(outc0(i) - x0(i), 8);

*	Total ouput
out0(i) = d0(i) + x0(i);

display int0, vld0, vkd0, outs0, outc0, x0, d0, out0;

chk_out_p(i,"sec") = -1;
chk_out_p(i,"com") = -1;
chk_out_p(i,"sec")$outs0(i) = 1;
chk_out_p(i,"com")$outc0(i) = 1;
display chk_out_p;

*	Gross value added.
vva0(i)$outs0(i) = ncapinc(i) + captax(i) + nlabinc(i) + labtax(i);
display vva0;

*	Labor tax rate.
tl0(i)$vld0(i) = labtax(i) / vld0(i);

*	Capital tax rate.
tk0(i)$vkd0(i) = captax(i) / vkd0(i);

*	Indirect tax rate.
tq0(i)$outs0(i) = nitax(i) / outs0(i);
display tl0, tk0, tq0;

*	----------------------------------------------------------------------
*	Investment
display "com: Investment";
parameter
    i0		    Total value of gross investment
    ishr(i)	    Share of each good in investment
    id0(i)	    Gross investment demand;

id0(i) = inve(i);
i0 = sum(i, id0(i));
ishr(i) = id0(i)/i0;
display id0, i0, ishr;

*	----------------------------------------------------------------------
*	Government demand
display "com: Government demand";
parameter
	gd0(i)  	Government demand (including gov investment)
	g0	    	Government total expenditure;

gd0(i) = gexp(i);
g0 = sum(i, gd0(i));
display gd0, g0;

*	----------------------------------------------------------------------
*	Trade and Armington.
display "com: Trade and Armington";
parameter
	a0(i)		Armington supply
	m0(i)		Demand for imported good (net of import tax)
	tm0(i)		Import tax rate
	bop0		Balance of payments of each good (ex - im)
	bopt0		Balance of payments (ex - im);

m0(i) = - impo(i);
tm0(i)$m0(i) = - impt(i)/m0(i);
a0(i) = d0(i) + (1+tm0(i)) * m0(i);
bop0(i) = x0(i) - m0(i);
bopt0 = round(sum(i, bop0(i)), 8);
display m0, tm0, a0, bop0, bopt0;

*	----------------------------------------------------------------------
*	Household data
display "com: Household data";

parameter
	epi0		Expanded disposal income
	wg0		Household gross current expenditure
	wn0		Household net current expenditure
	c0		Household consumption expenditure
	ce0		Energy good consumption
	cne0		Non-energy good consumption
	cg0		Household gross consumption expenditure
	cd0		Household consumption demand

	li0		Value of labor income (gross of tax)
	ki0		Value of capital income (gross of tax)

	tli0		labor income tax rate
	tci0		Capital income tax rate
	ts0		Consumption subsidy rate
	tc0		Consumption tax rate

	vos		Value of savings
	vot		Value of net transfer
;
li0 = lincome;
ki0 = cincome;
vos = hsave;
vot = hntrans;
cd0(i) = cons(i) + csub(i);
display li0, ki0, vos, cd0, vot;

ts0(i)$cd0(i) = - csub(i) / cd0(i);
tc0(i)$cons(i) = ctax(i) / cons(i);
c0 = sum(i, cons(i));
cg0 = sum(i, (1+tc0(i)) * (1+ts0(i)) * cd0(i));
tli0 = linctax / li0;
tci0 = cinctax / ki0;
display ts0, tc0, cg0, tli0, tci0;

cne0 = sum(nene, cons(nene));
ce0 = sum(ene, cons(ene));
display cne0, ce0;

*	----------------------------------------------------------------------
*	Total labor endowment.
display "com: Total labor endowment.";
$ontext

Total labor endowment = a * labor supply

Leisure = total labor endowment - labor supply

$BAmO+F/J]M-NL$r$b$H$a$k!#(B

$B<}F~3,5iJLO+F/;~4V?t?d7W$G$O!"$I$N3,5i$b7nJ?6QO+F/;~4V?t$O$[$\(B 190 $B;~4V!#0l7n$K(B
$BO+F/!"$*$h$SM>2K$N$?$a$K;HMQ2DG=$J;~4V?t$O!"0lF|Ev$j(B 16 $B;~4V$H$9$k$H!"(B16x30 =
480 $B$h$j!"(B480 $B;~4V!#(B

$B$3$l$h$j!"(B

a = $B;HMQ2DG=$JAm;~4V?t(B / $BO+F/;~4V?t(B = 480 / 190 = 2.53

$offtext
parameter
	lratio	    Labor supply ratio		    / 2.53 /
	endl	    Benchmark total labor endowment
	lei0	    Benchmark leisure;

endl = lratio * li0;
lei0 = endl - li0;
display lratio, endl, lei0;

*	----------------------------------------------------------------------
*	Defining sets and parameters for dynamic model.
parameter
	bopcon		Balance of payments restriction / %bopcon% /
	capflo		Free capital flow
	phi		Adjustment speed		/ %phi% /;

*	Capital flow constraint: If balance of payment restriction is not
*	imposed, capital flow is free.

bopcon = 1$%bopcon%;
capflo = yes$(not bopcon);

display bopcon, capflo, phi;

*	-------------------------------------------------------------------
*	Calibrate base year variables.
$ontext
Calibrate benchmark and baseline variables to make the baseline equilibrium
the steady state.

$offtext
parameter
	vks0	    Benchmark total value of capital stock
	vai0	    Benchamrk value of capital income (net of tax)
	vat0	    Benchmark capital income tax;

vks0 = sum(i, vkd0(i));
vat0 = cinctax;
vai0 = cincome - vat0;
display vks0, vat0, vai0;

*	Calibrated parameters
parameter
	r		Baseline interest rate
	gr		Baseline growth rate (labor)
	delta		Depreciation rate
	j0		Benchmark net investment
	jd0		Benchmark net investment demand

	rk0		Benchmark rental price for producer
	rke0		Benchmark rental price for household
	ks0		Benchmark capital stock
	pk0		Benchmark capital price
	pka0		Reduced cost of investment

	ld0		Benchmark labor input (quantity)
	kd0		Benchmark capital input (quantity);

*	Target values for calibrated variables.
parameter
	rt0		Interest rate (target level)		/ 0.03 /
	delt0		Depreciation rate (target level)	/ 0.07 /
	grt0		Labor growth rate (target level)	/ 0.01 /;
display rt0, delt0, grt0;

*	Variable definitions.
variables
	rt		Interest rate
	delt		Depreciation rate
	rket		Rental rate for household
	kt		Capital stock

	jt		Net investment
	loss		Loss

	lam_1
	lam_2
	lam_3;

*	Equation definitions.
equations
	eq_obj	    Objetive function
	eq_con1	    Constraint 1
	eq_con2	    Constraint 2
	eq_con3	    Constraint 3
	eq_con4	    Constraint 4

	eq_rt	    FOC
	eq_delt	    FOC
	eq_rket	    FOC
	eq_kt	    FOC;

*	----------------------------------------------------------------------
*	NLP model.
eq_obj .. loss =e=
	  power((rt - rt0)/rt0, 2) + power((delt - delt0)/delt0, 2);

eq_con1 .. rket =e= (1+phi*(grt0+delt))*(rt+delt)
	   - phi*(delt+grt0)**2 / 2;

eq_con2 .. rket*kt =e= vai0;

eq_con3 .. i0 =e= (delt+grt0) * kt * (1+phi*(delt+grt0)/2);

model calib_nlp NLP model / eq_obj, eq_con1, eq_con2, eq_con3 /;

*	----------------------------------------------------------------------
*	MCP model.
eq_rt .. 2*(rt-rt0)/(rt0**2) - lam_1*(1+phi*(grt0+delt)) =e= 0;

eq_delt .. 2*(delt-delt0)/(delt0**2)
	   + lam_1*( phi*(grt0-rt) - (1+phi*(grt0+delt)) )
	   - lam_3*kt*(1+phi*(delt+grt0)) =e= 0;

eq_rket .. lam_1 + lam_2*kt =e= 0;

eq_kt .. lam_2*rket - lam_3*(delt+grt0)*(1 + phi*(delt+grt0)/2) =e= 0;

model calib_mcp MCP model / eq_con1, eq_con2, eq_con3, eq_rt, eq_delt,
			    eq_rket, eq_kt /;

*	Initial values.
rt.l = rt0;
delt.l = delt0;
kt.l = vks0 / (delt.l + rt.l);
rket.l = rt.l+delt.l;

lam_1.l = 0;
lam_2.l = 0;
lam_3.l = 0;

parameter	calibres	Calibrted results;

if(sol_mcp,
    display "com: Calibration by MCP";
    solve calib_mcp using mcp;
    calibres("r","cal") = rt.l;
    calibres("rke","cal") = rket.l;
    calibres("delta","cal") = delt.l;
    calibres("gr","cal") = grt0;
    calibres("k0","cal") = kt.l;
    calibres("pk0","cal") = (1+rt.l)*(1+phi*(delt.l+grt0));
    calibres("vks0","cal") = calibres("k0","cal") * calibres("rke","cal");
    calibres("j0","cal") = (delt.l+grt0)*kt.l;
    calibres("pka0","cal") = phi*(delt.l+grt0)**2 / 2;
    calibres("adjc","cal") = phi*(delt.l+grt0) / 2;
    display calibres;
    else
    display "com: Calibration by NLP";
    solve calib_nlp using nlp minimizing loss;
    calibres("r","cal") = rt.l;
    calibres("rke","cal") = rket.l;
    calibres("delta","cal") = delt.l;
    calibres("gr","cal") = grt0;
    calibres("k0","cal") = kt.l;
    calibres("pk0","cal") = (1+rt.l)*(1+phi*(delt.l+grt0));
    calibres("vks0","cal") = calibres("k0","cal") * calibres("rke","cal");
    calibres("j0","cal") = (delt.l+grt0)*kt.l;
    calibres("pka0","cal") = phi*(delt.l+grt0)**2 / 2;
    calibres("adjc","cal") = phi*(delt.l+grt0) / 2;
    display calibres;
);

r = rt.l;
gr = grt0;
delta = delt.l;
rke0 = rket.l;
rk0 = rke0/(1-tci0);
ks0 = kt.l;
pk0 = calibres("pk0","cal");
pka0 = calibres("pka0","cal");
j0 = calibres("j0","cal");
display "Display calibrated parameters";
display r, gr, delta, rke0, rk0, ks0, pk0, pka0, j0;

*	Benchmark capital inputs.
kd0(i) = ks0*vkd0(i)/vks0;
display kd0;

*	Benchmark net investment demand.
jd0(i) = ishr(i)*j0;
display jd0;

*	wrt labor, quantity = value.
ld0(i) = vld0(i);
display ld0;

*	----------------------------------------------------------------------
*	Reference prices.
display "com: Reference prices.";

parameter
	pl0		Reference wage rate for industry
	rk00		Reference rental price for industry
	rk000		Reference rental price for industry
	rkx0
	rkxe0

	ple0		Reference labor price for household (price of leisure)
	rke0		Reference rental price for household
	pm0		Reference price of imported goods
	pc0		Reference good price for household
	pc00		Reference good price for household

	pd0		Reference price for domestic supply
	px0		Reference price for export supply
	pq0		Reference price for output;

pl0(i) = 1+tl0(i);
rk00(i) = (1+tk0(i))*rk0;
rk000(i) = (1+tk0(i));

ple0 = 1-tli0;
rke0 = (1-tci0)*rk0;

pc0(i) = (1+ts0(i));
pc00 = 1.03;
pm0(i) = 1+tm0(i);

px0(i) = 1-tq0(i);
pd0(i) = 1-tq0(i);
pq0(i) = 1-tq0(i);

display pl0, rk00, ple0, rke0, pc0, pm0, px0, pd0;

$ontext
expenditure $B$O(B gross $B$+(B net $B$+!)(B
$offtext
*	Benchmark current expenditure.
wg0 = cg0 + ple0*lei0;
display wg0;

*	Benchmark expanded disposal income = labor income + capital income -
*	saving.
epi0 = ple0*endl + rke0*ks0 - vos + vot;
display epi0;

parameter	chk_revex	Check revenue and expenditure consistency;
chk_revex("rev-exp") = round(wg0 - epi0, 3);
display chk_revex;
abort$chk_revex("rev-exp") "Household revenue and expenditure are inconsistent";

*	----------------------------------------------------------------------
*	Baseline path.
display "com: Baseline path.";

parameter
    qref(t)	    Reference quantity path
    pref(t)	    Reference price path
    wref(t)	    Reference (period) utility path;

*	Reference quantity path.
qref(t) = (1+gr)**(ord(t)-1);

*	Reference price path.
pref(t) = (1/(1+r))**(ord(t)-1);

*	Reference utility path
wref(t) = wg0 * qref(t);

display qref, pref, wref;

parameter
	gqref(t)	Reference government expenditure
	gbar		Reference government expenditure;
gqref(t) = g0*qref(t);
gbar = sum(t, pref(t) * gqref(t));
display gqref, gbar;

parameter
	ubar	    Benchmark lifetime utility;

*	Prsigent value of utility over the model horizon:
ubar = sum(t, wref(t) * pref(t));
display ubar;

parameter	adj0;

* adj0 = pka0*phi*(delta+gr)/2 + sum(i, ishr(i)*phi*(delta+gr)/2);
adj0 = phi*(delta+gr)/2 + sum(i, ishr(i)*phi*(delta+gr)/2);
display adj0;

*	----------------------------------------------------------------------
*	Baseline tax rate.
display "com: Baseline tax rate..";
parameter
    tq(i)		Indirect tax rate
    tm(i)		Import tax rate
    tl(i)		Tax rate on labor input in production
    tk(i)		Tax rate on capital input in production

    tli			Labor income tax rate
    tci			Capital income tax rate
    ts(i)		Consumption subsidy rate
    tc			Consumption tax rate
    tsc(t,i)

    tca(t,e)		Carbon tax rate;

*	Initial values.
tq(i) = tq0(i);
tm(i) = tm0(i);
tl(i) = tl0(i);
tk(i) = tk0(i);
tc = 0.03;
tli = tli0;
tci = tci0;
ts(i) = ts0(i);
tca(t,e) = 0;
tsc(t,i) = tc0(i) + ts0(i) + tc0(i) * ts0(i);
display tq, tm, tl, tk, tc, tli, tci, ts, tca, tsc;

*	----------------------------------------------------------------------
*	Calibrate leisure-consumtpion substitution.
display "com: Calibrate leisure-consumtpion substitution.";

sigcl = ( eols * li0 / lei0 + ple0 * li0 / epi0 )
    / (1 - ple0 * lei0 / epi0 );

display eols, sigcl;

*	Check calibration.
parameter sigcl_, sh_l, sh_le, sh_led;

sh_l = ple0*li0 / epi0;
sh_led = ple0*endl / epi0;
sh_le = ple0*lei0 / epi0;
sigcl_("sigcl") = ( eols*sh_l/(sh_led - sh_l) + sh_l ) / (1 - sh_le);
sigcl_("diff") = round(sigcl_("sigcl") - sigcl, 6);
display sh_l, sh_le, sh_led, sigcl_;
abort$sigcl_("diff") "Inconsistent in calibration";

*	----------------------------------------------------------------------
*	Check data.
display "com: Check data.";

*	Check final demand and value added.
chk_fdva(i,"va") = rk00(i)*kd0(i) + pl0(i)*ld0(i)
    + tq0(i)*outs0(i);
chk_fdva("sum","va") = round(sum(i, chk_fdva(i,"va")), 6);

chk_fdva(i,"fd") = cd0(i) + id0(i) + gd0(i) + bop0(i) - tm0(i)*m0(i);
chk_fdva("sum","fd") = round(sum(i, chk_fdva(i,"fd")), 6);

chk_fdva("diff","fd-va") = round(chk_fdva("sum","fd") - chk_fdva("sum","va"), 6);
display chk_fdva;
abort$chk_fdva("diff","fd-va") "FD and VA are inconsistent.";

*	Check output data.
chk_out(i,"sec") = sum(ii, int0(ii,i)) + chk_fdva(i,"va");
chk_out(i,"com") = sum(ii, int0(i,ii)) + chk_fdva(i,"fd");
chk_out("sum","sec") = sum(i, chk_out(i,"sec"));
chk_out("sum","com") = sum(i, chk_out(i,"com"));
chk_out(i,"sec_com") = round(chk_out(i,"sec") - chk_out(i,"com"), 6);
chk_out("sum","sec_com") = round(chk_out("sum","sec") - chk_out("sum","com"), 6);
display chk_out;

abort$chk_out("sum","sec_com") "Output data are inconsistent.";

*	----------------------------------------------------------------------
*	Check investment and savings consistency.
parameter chk_is	check investment and saving;

chk_is("save") = vos;
chk_is("ginv") = i0;
*	Saving = investment because benchmark balance of payment and
*	government fiscal balance are zero.
chk_is("save-ginv") = round(chk_is("save") - chk_is("ginv") - bopt0, 6);
display chk_is;
abort$chk_is("save-ginv") "Inconsistent";

*	----------------------------------------------------------------------
*	Check household income and expenditure.
parameter chk_ie	Check household income and expenditure;
*	Expanded income.
chk_ie("inc") = ple0*endl + rke0*ks0 - vos + vot;
*	Expanded expenditure.
chk_ie("exp") = cg0 + ple0*lei0;
chk_ie("inc-exp") = round(chk_ie("inc") - chk_ie("exp"), 6);
display chk_ie;
abort$chk_ie("inc-exp") "Income and expenditure are inconsistent!";

*	----------------------------------------------------------------------
*	Check government revenue and expenditure
parameter chk_gov	Check government revenue and expenditure;

* 	Government revenue.
chk_gov("rev")
    = sum(i, tk0(i) * rk0 * kd0(i)
    + tl0(i) * ld0(i)
    + tq0(i) * (x0(i)+d0(i)))
    + tli0 * li0
    + tci0 * rk0 * ks0
    + sum(i, tm0(i) * m0(i))
    + sum(i, ts0(i) * cd0(i))
    + sum(i, tc0(i) * (1 + ts0(i)) * cd0(i))
    - vot;
chk_gov("exp")= g0;
chk_gov("rev-exp") = round(chk_gov("rev") - chk_gov("exp"), 6);
display chk_gov;
abort$chk_gov("rev-exp") "Government expenditure and revenue are inconsistent";

*	----------------------------------------------------------------------
*	Adjust energy (quantity) data.
display "com: Adjust energy (quantity) data.";

*	----------------------------------------------------------------------
*	Check imported energy data.
display "com: Check imported energy data.";

display ioqc, ioqnc, fdq;

parameter cacoef Carbon coefficient (t-C|10^7kcal)/
*=>gams2prm cacoef
* Called from c:\home\cream\gams\double-dividend-jp\data-new\io95_to_35.gms, line 65343
* 11/29/03 14:54:05
lim   1.2000000000000E-01
coc   1.0451149310000E+00
sla   1.0154523000000E+00
cru   7.9152314900000E-01
nat   5.8507665700000E-01
pet   7.8289055874027E-01
cok   1.2310959200000E+00
gas   5.9652317200000E-01
/;

parameter ioca	Carbon emissions (MtC);

*	Calculate carbon emissions.
ioca(e,i) = cacoef(e) * ioqc(e,i);
ioca(e,"cons") = cacoef(e) * fdq(e,"cons");

ioca(e,"end") = sum(i, ioca(e,i));
ioca(e,"fd") = ioca(e,"cons");
ioca(e,"sum") = ioca(e,"end") + ioca(e,"fd");

ioca("sum",i) = sum(e, ioca(e,i));
ioca("sum","cons") = sum(e, ioca(e,"cons"));
ioca("sum","sum") = sum(e, ioca(e,"sum"));
display ioca;

*	----------------------------------------------------------------------
*	Check consistency between value and quantity data.
display "com: Check consistency between value and quantity data.";
$ontext
$B6b3[%G!<%?$H?tNL%G!<%?$N@09g@-$r%A%'%C%/$9$k!%(B

$offtext
*	----------------------------------------------------------------------
*	Check the value and quantity consistency
display "com: Check the value and quantity consistency";
$ontext
value data $B$H(B quantity data $B$N@09g@-(B ($BId9f$,$"$C$F$$$k$+(B) $B$r%A%'%C%/!%(B

3 $B$D$N%1!<%9$,$"$j$&$k!%(B

(1) Value $B$b(B quantity $B$b@5!%(B
(2) Value $B$O@5$@$,(B quantity $B$O%<%m!%(B
(3) Value $B$O%<%m$@$,(B quantity $B$O@5!%(B

(1) $B$OLdBj$J$$!%(B(2) $B$H(B (3) $B$N>l9g$O!$(Bquantity $B$r%<%m$H$9$k!%$D$^$j!$$=$3$+$i$N(B
$BGS=PNL$O$J$$$b$N$H$9$k!%(B

$B$3$ND4@0$K$h$jGS=PNL$O2a>.$H$J$k!%(B
$offtext
parameter
	ch_iov
	ch_iovc
	ch_iovnc

	ch_ioq
	ch_ioqc		Int input for combustion
	ch_ioqnc	Int input for non-combustion
	ch_fdq		Final demand (quantity)

	ch_flag		Flag for check: (0 or 1)
	ch_flag_c	Flag for check: (0 or 1)
	ch_flag_nc	Flag for check: (0 or 1)
	ch_flag_fd	Flag for check: (0 or 1);

*	Check sign consistency.
ch_iov(e,i) = iov(e,i);
ch_iovc(e,i) = iovc(e,i);
ch_iovnc(e,i) = iovnc(e,i);

ch_ioq(e,i) = ioq(e,i);
ch_ioqc(e,i) = ioqc(e,i);
ch_ioqnc(e,i) = ioqnc(e,i);

ch_flag(e,i) = 0;
ch_flag_c(e,i) = 0;
ch_flag_nc(e,i) = 0;

*	Intermediate inputs.
loop((e,i)$(ch_iov(e,i) or ch_ioq(e,i)),
	if((ch_iov(e,i) * ch_ioq(e,i) > 0),
		ch_flag(e,i) = 0;
		elseif (ch_iov(e,i) * ch_ioq(e,i) = 0),
		ch_flag(e,i)$ch_iov(e,i) = 1;
		ch_flag(e,i)$ch_ioq(e,i) = -1;
	));
option ch_flag:0;
display ch_flag;

*	Combustion purpose.
loop((e,i)$(ch_iovc(e,i) or ch_ioqc(e,i)),
	if((ch_iovc(e,i) * ch_ioqc(e,i) > 0),
		ch_flag_c(e,i) = 0;
		elseif (ch_iovc(e,i) * ch_ioqc(e,i) = 0),
*		Value data only.
		ch_flag_c(e,i)$ch_iovc(e,i) = 1;
		ioqc(e,i)$ch_iovc(e,i) = 0;
*		Quantity data only.
		ch_flag_c(e,i)$ch_ioqc(e,i) = -1;
		ioqc(e,i)$ch_ioqc(e,i) = 0;
	));

*	Non-combustion purpose.
loop((e,i)$(ch_iovnc(e,i) or ch_ioqnc(e,i)),
	if((ch_iovnc(e,i) * ch_ioqnc(e,i) > 0),
		ch_flag_nc(e,i) = 0;
		elseif (ch_iovnc(e,i) * ch_ioqnc(e,i) = 0),
*		Value data only.
		ch_flag_nc(e,i)$ch_iovnc(e,i) = 1;
		ioqnc(e,i)$ch_iovnc(e,i) = 0;
*		Quantity data only.
		ch_flag_nc(e,i)$ch_ioqnc(e,i) = -1;
		ioqnc(e,i)$ch_ioqnc(e,i) = 0;
	));

option ch_flag_c:0, ch_flag_nc:0;
display ch_flag_c, ch_flag_nc;

display "After adjustment", ioqc, ioqnc;

*	Calculate carbon emissions.
ioca(e,i) = cacoef(e) * ioqc(e,i);
ioca(e,"cons") = cacoef(e) * fdq(e,"cons");

ioca(e,"end") = sum(i, ioca(e,i));
ioca(e,"fd") = ioca(e,"cons");
ioca(e,"sum") = ioca(e,"end") + ioca(e,"fd");

ioca("sum",i) = sum(e, ioca(e,i));
ioca("sum","cons") = sum(e, ioca(e,"cons"));
ioca("sum","sum") = sum(e, ioca(e,"sum"));
display ioca;

*	----------------------------------------------------------------------
*	Contribution ratio.
display "com: Contribution ratio";
parameter	rat_ene		Contribution ratio;

rat_ene(e,i)$(ioqc(e,i)+ioqnc(e,i))
    = ioqc(e,i)/(ioqc(e,i)+ioqnc(e,i));
display rat_ene;

*	----------------------------------------------------------------------
*	Define energy inputs and consumption.
display "com: Define energy inputs and consumption.";
$ontext

$offtext
parameter
	veic		Value of energy input (combustion purpose)
	veict		Sumal value of energy input (combustion purpose)
	veinc		Value of energy input (non-combustion purpose)
	vec		Value of energy consumption
	vect		Value of energy for combustion purpose;

veic(e,i) = rat_ene(e,i) * int0(e,i);
veic("ele",i) = int0("ele",i);
veic(e,"sum") = sum(i, veic(e,i));
veinc(e,i) = int0(e,i) - veic(e,i);
display veic, veinc;

vec(e) = cd0(e);
display vec;

veict(i) = sum(ec, veic(ec,i)) + veic("ele",i);
display veict;

vect(e) = veic(e,"sum") + vec(e);
display vect;

*	Value of energy inputs
ve0(i)$outs0(i) = sum(ec, veic(ec,i)) + sum(ele, int0(ele,i));
display ve0;

*	VA and energy.
vvae0(i)$outs0(i) = vva0(i) + ve0(i);
display vvae0;

*	----------------------------------------------------------------------
*	Deriving carbon emission data.
display "com: Deriving carbon emission data.";

parameter
	carbi0(*,*)	Carbon emissions from intermediate inputs (MtC)
	carbc0(*)	Carbon emissions from final consumption (MtC)
	carbt0		Carbon emissions (MtC)

	carbl		Carbon emission restriction (MtC)
	carbl0		Benchmark carbon emissions (MtC)

	scale_ca	Scale factor / 1000 /
;
*	Carbon emissions from intermediate inputs.
carbi0(e,i) = cacoef(e)*ioqc(e,i) / scale_ca;
carbi0(e,"sum") = sum(i, carbi0(e,i));
carbi0("sum",i) = sum(e, carbi0(e,i));
carbi0("sum","sum") = sum((e,i), carbi0(e,i));

*	Carbon emissions from final demand (consumption).
carbc0(e) = cacoef(e)*fdq(e,"cons") / scale_ca;
carbc0("sum") = sum(e, carbc0(e));
display carbi0, carbc0;

carbt0(e) = carbi0(e,"sum") + carbc0(e);
carbt0("sum") = sum(e, carbt0(e));
display carbt0;

*	Benchmark carbon emissions.
carbl0 = sum((e,i), carbi0(e,i)) + sum(e, carbc0(e));
*	Carbon emission restriction is set to zero tentatively.
* carbl(t) = 0;
carbl(t) = carbl0 * 1e+6;
option carbl0:6, carbl:6;
display carbl0, carbl;

parameter	chk_carbon	Check carbon emissions (MtC);

*	Carbon emissions from intermediate inputs.
chk_carbon(e,i) = carbi0(e,i);
chk_carbon(e,"int") = sum(i, chk_carbon(e,i));
chk_carbon("sum","int") = sum(e, chk_carbon(e,"int"));
chk_carbon("sum",i) = sum(e, chk_carbon(e,i));
*	Carbon emissions from consumption.
chk_carbon(e,"con") = carbc0(e);
chk_carbon("sum","con") = sum(e, chk_carbon(e,"con"));
chk_carbon(e,"sum") = chk_carbon(e,"int") + chk_carbon(e,"con");
*	Total carbon emissions.
chk_carbon("sum","sum") = sum(e, chk_carbon(e,"sum"));
option chk_carbon:3;
display chk_carbon;

*	----------------------------------------------------------------------
*	Emission source consumption
display "com:************************************************";
display "com: Emission source consumption";
display "com:************************************************";

parameter
	es_cons		Emission source consumption (10**13kcal)
	es_consa	Emission source consumption aggregated (10**13kcal)
	es_conc		Emission source consumption for combustion purpose (10**13kcal)
	es_connc	Emission source consumption for non-combustion purpose (10**13kcal)
	es_const	Emission source total (10**13kcal)
	es_share	Emission source share (\%)
	es_sharea	Emission source share aggregated (\%)
;
*	Emission source consumption of each sector.
es_cons(ai,es)
	= round(sum(i$map_ai(i,ai), ioqc(es,i) + ioqnc(es,i)), 2);
es_cons(ai,"tot") = sum(es, es_cons(ai,es));
es_cons("int",es) = sum(ai, es_cons(ai,es));
es_cons("int","tot") = sum((ai,es), es_cons(ai,es));
es_cons("fcon",es)
	= fdq(es,"cons");
es_cons("fcon","tot") = sum(es, es_cons("fcon",es));
es_cons("tot",es) = es_cons("int",es) + es_cons("fcon",es);
es_cons("tot","tot") = sum(ai, es_cons(ai,"tot")) + es_cons("fcon","tot");
option decimals = 2;
display es_cons;

file fout / .\benchmark\benchmark_data.tex /;
put fout;
* $setglobal debug on
$setglobal format tex
$batinclude gams2tbl es_cons

*	Emission source consumption for combusted use
es_conc(ai,es)
	= round(sum(i$map_ai(i,ai), ioqc(es,i)), 2);
es_conc(ai,"tot") = sum(es, es_conc(ai,es));
es_conc("int",es) = sum(ai, es_conc(ai,es));
es_conc("int","tot") = sum((ai,es), es_conc(ai,es));
es_conc("fcon",es)
	= fdq(es,"cons");
es_conc("fcon","tot") = sum(es, es_conc("fcon",es));
es_conc("tot",es) = es_conc("int",es) + es_conc("fcon",es);
es_conc("tot","tot") = sum(ai, es_conc(ai,"tot")) + es_conc("fcon","tot");
option decimals = 2;
display es_conc;
$batinclude gams2tbl es_conc

*	ES consumption for non combusted purpose
es_connc(ai,es)
	= round(sum(i$map_ai(i,ai), ioqnc(es,i)), 2);
option decimals = 2;
display es_connc;
$batinclude gams2tbl es_connc

*	Energy share.
es_share(ai,es)
	= round(100*( es_cons(ai,es) )
	/ ( es_cons("tot",es) ), 1);
es_share("int",es)
	= round(100*( es_cons("int",es) )
	/ ( es_cons("tot",es) ), 1);
es_share("fcon",es)
	= round(100*( es_cons("fcon",es) )
	/ ( es_cons("tot",es) ), 1);
es_share(ai,"tot")
	= round(100*( es_cons(ai,"tot") )
	/ ( es_cons("tot","tot") ), 1);
es_share("int","tot")
	= round(100*( es_cons("int","tot") )
	/ ( es_cons("tot","tot") ), 1);
es_share("fcon","tot")
	= round(100*( es_cons("fcon","tot") )
	/ ( es_cons("tot","tot") ), 1);
es_share("tot",es)
	= round(sum(ai, es_share(ai,es)) + es_share("fcon",es), 1);
es_share("tot","tot")
	= round(sum(ai, es_share(ai,"tot")) + es_share("fcon","tot"), 1);
option es_share:1;
display es_share;
$batinclude gams2tbl es_share

*	----------------------------------------------------------------------
*	Share of emission sources
display "com: Share of emission sources";

parameter
	es_sh_ene	Share of emission sources (combustion purpose) (\%)
	es_sh_enea	Share of each emission source (combustion) (\%);

es_sh_ene(es,"int")
	= round(100*( es_conc("int",es) / es_conc("int","tot") ), 1);
es_sh_ene(es,"fcon")
	= round(100*( es_conc("fcon",es) / es_conc("fcon","tot") ), 1);
es_sh_ene(es,"tot")
	= round(100*( es_conc("tot",es) / es_conc("tot","tot") ), 1);
es_sh_ene("tot","int") = sum(es, es_sh_ene(es,"int"));
es_sh_ene("tot","fcon") = sum(es, es_sh_ene(es,"fcon"));
es_sh_ene("tot","tot") = sum(es, es_sh_ene(es,"tot"));
option es_sh_ene:1;
display es_sh_ene;
$batinclude gams2tbl es_sh_ene
* $batinclude xldump es_sh_ene .\benchmark\es_sh_ene.xls

*	----------------------------------------------------------------------
*	Emission Source input share in each sector.
display "com: Emission Source input share in each sector.";

parameter
	e_sh_s	Emission Source input share in each sector
	e_sh_sa	Emission Source input share in each sector (aggregated);

e_sh_s(i,"share(\%)")$outs0(i)
	= round(100*sum(e, veic(e,i)) / outs0(i), 1) + eps;
e_sh_sa(ai,"share(\%)")
	= round(100 * sum(i$map_ai(i,ai), sum(e, veic(e,i)))
	/ sum(i$map_ai(i,ai), outs0(i)), 1) + eps;
option e_sh_s:1;
display e_sh_s;
option e_sh_sa:1;
display e_sh_sa;
$batinclude gams2tbl e_sh_s
$batinclude gams2tbl e_sh_sa

*	----------------------------------------------------------------------
*	Carbon emissions data.
display "com: Carbon emissions data.";

parameter
	ca_emi		Carbon emissions (MtC)
	ca_emia		Carbon emissions aggregated (MtC)
	ca_emia_	Carbon emissions aggregated (MtC)
	ca_share	Emissions share (\%)
	ca_share_ene	Emissions share of each source (\%)
	ca_share_sec	Emissions share of sector (\%)
	ca_sharea_ene	Emissions share of each source aggregated (\%)
;
*	Carbon emissions from each sector.
ca_emi(ai,e) = sum(i$map_ai(i,ai), cacoef(e)*ioqc(e,i));
ca_emi("int",e) = sum(ai, ca_emi(ai,e));
ca_emi(ai,"tot") = sum(e, ca_emi(ai,e));
ca_emi("int","tot") = sum((ai,e), ca_emi(ai,e));
ca_emi("fcon",e) = cacoef(e)*fdq(e,"cons");
ca_emi("fcon","tot") = sum(e, ca_emi("fcon",e));
ca_emi("tot",e) = ca_emi("int",e) + ca_emi("fcon",e);
ca_emi("tot","tot") = ca_emi("int","tot") + ca_emi("fcon","tot");

option ca_emi:4;
display ca_emi;

*	Carbon emissions from each sector (aggregated).
ca_emia(ai,ae) = sum(i$map_ai(i,ai), sum(e$map_ae(e,ae), cacoef(e)*ioqc(e,i)));
ca_emia("int",ae) = sum(ai, ca_emia(ai,ae));
ca_emia(ai,"tot") = sum(ae, ca_emia(ai,ae));
ca_emia("int","tot") = sum((ai,ae), ca_emia(ai,ae));
ca_emia("fcon",ae) = sum(e$map_ae(e,ae), cacoef(e)*fdq(e,"cons"));
ca_emia("fcon","tot") = sum(ae, ca_emia("fcon",ae));
ca_emia("tot",ae) = ca_emia("int",ae) + ca_emia("fcon",ae);
ca_emia("tot","tot") = ca_emia("int","tot") + ca_emia("fcon","tot");
option ca_emia:2;
display ca_emia;
$batinclude gams2tbl ca_emia

*	Carbon emissions from each sector (aggregated).
ca_emia_(ai,ae) = sum(i$map_ai(i,ai), sum(e$map_ae(e,ae), cacoef(e)*ioqc(e,i)));
ca_emia_("fcon",ae) = sum(e$map_ae(e,ae), cacoef(e)*fdq(e,"cons"));
option ca_emia_:3;
display ca_emia_;
* $batinclude xldump ca_emia_ .\benchmark\ca_emia.xls

*	Carbon share of each energy.
ca_share(ai,e) = 100*ca_emi(ai,e)
	/ ca_emi("tot",e);
ca_share("fcon",e) = 100*ca_emi("fcon",e)
	/ ca_emi("tot",e);
ca_share("tot","tot") = sum((ai,e), ca_share(e,ai));
option ca_share:3
display ca_share;

*	Carbon share of each energy.
ca_share_ene(e,"int")
	= round(100*ca_emi("int",e)
	/ ca_emi("int","tot"), 1);
ca_share_ene(e,"fcon")
	= round(100*ca_emi("fcon",e)
	/ ca_emi("fcon","tot"), 1);
ca_share_ene(e,"tot")
	= round(100*ca_emi("tot",e)
	/ ca_emi("tot","tot"), 1);
ca_share_ene("tot","int") = sum(e, ca_share_ene(e,"int"));
ca_share_ene("tot","fcon") = sum(e, ca_share_ene(e,"fcon"));
ca_share_ene("tot","tot") = sum(e, ca_share_ene(e,"tot"));
option ca_share_ene:1;
display ca_share_ene;
$batinclude gams2tbl ca_share_ene
* $batinclude xldump ca_share_ene .\benchmark\ca_share_ene.xls

*	Carbon share of each sector
ca_share_sec(ai,"share")
	= round(100*ca_emi(ai,"tot")
	/ ca_emi("tot","tot"), 1);
ca_share_sec("fcon","share") = 100*ca_emi("fcon","tot")
	/ ca_emi("tot","tot");
option ca_share_sec:1;
display ca_share_sec;
$batinclude gams2tbl ca_share_sec

*	Carbon share of each energy and sector (aggregated).
ca_sharea_ene(ae,"int") = 100*ca_emia("int",ae)
	/ ca_emia("int","tot");
ca_sharea_ene(ae,"fcon") = 100*ca_emia("fcon",ae)
	/ ca_emia("fcon","tot");
ca_sharea_ene(ae,"tot") = 100*ca_emia("tot",ae)
	/ ca_emia("tot","tot");
ca_sharea_ene("tot","int") = sum(ae, ca_sharea_ene(ae,"int"));
ca_sharea_ene("tot","fcon") = sum(ae, ca_sharea_ene(ae,"fcon"));
ca_sharea_ene("tot","tot") = sum(ae, ca_sharea_ene(ae,"tot"));
option ca_sharea_ene:1
display ca_sharea_ene;

*	----------------------------------------------------------------------
*	Carbon intensity
display "com: Carbon intensity";

parameter
	ca_int		"Carbon intensity (tC / mil_yen)"
	ca_inta		"Carbon intensity aggregated (tC / mil_yen)"
	ca_outv		Value of output
	ca_outva	Value of output
;
ca_outv(i) = outs0(i);
ca_int(i,"cint")$ca_outv(i)
	= sum(e, cacoef(e)*ioqc(e,i))
	/ ca_outv(i);
ca_outva(ai) = sum(i$map_ai(i,ai), outs0(i));
ca_inta(ai,"cint")$ca_outva(ai)
	= ca_emi(ai,"tot") / ca_outva(ai);
option ca_int:2;
display ca_int;
option ca_inta:2;
display ca_inta;
$batinclude gams2tbl ca_inta

*	----------------------------------------------------------------------
*	Tax data.
display "com: Tax data.";

parameter
	cap_tax		Capital tax on production (bil yen)
	lab_tax		Labor tax on production (bil yen)
	linc_tax	Labor income tax (tril yen)
	cinc_tax	Capital income tax (tril yen)
	cons_tax	Consumption tax (tril yen)

	t_rev		Total tax revenue (bil yen)

	tax_prod	Tax on production
	tax_house	Tax on household
;
*	Corporate tax.
cap_tax(i,"rate") = 100*tk0(i);
cap_tax(i,"value_bil") = 1000*captax(i);
cap_tax(i,"net_cap_pay") = 1000*vkd0(i);
cap_tax("tot","net_cap_pay") = sum(i, cap_tax(i,"net_cap_pay"));
cap_tax("tot","value_bil") = sum(i, cap_tax(i,"value_bil"));
cap_tax("ave","rate") = 100*cap_tax("tot","value_bil")/cap_tax("tot","net_cap_pay");
cap_tax("var","rate")
    = sum(i, power(cap_tax(i,"rate") - cap_tax("ave","rate"),2))/card(i);
cap_tax("sd","rate") = sqrt(cap_tax("var","rate"));
option cap_tax:2
display cap_tax;

*	Social security
lab_tax(i,"rate") = 100*tl0(i);
lab_tax(i,"value_bil") = 1000*labtax(i);
lab_tax(i,"net_lab_pay") = 1000*vld0(i);
lab_tax("tot","value_bil") = sum(i, lab_tax(i,"value_bil"));
lab_tax("tot","net_lab_pay") = sum(i, lab_tax(i,"net_lab_pay"));
lab_tax("ave","rate") = 100*lab_tax("tot","value_bil")/lab_tax("tot","net_lab_pay");
lab_tax("var","rate")
    = sum(i, power(lab_tax(i,"rate") - lab_tax("ave","rate"),2))/card(i);
lab_tax("sd","rate") = sqrt(lab_tax("var","rate"));
option lab_tax:2
display lab_tax;

*	Summary for tax on production.
tax_prod(i,"cap_v") = round(cap_tax(i,"value_bil"), 1);
tax_prod("sum","cap_v") = sum(i, tax_prod(i,"cap_v"));
tax_prod(i,"cap_r") = cap_tax(i,"rate");
tax_prod(i,"lab_v") = round(lab_tax(i,"value_bil"), 1);
tax_prod("sum","lab_v") = sum(i, tax_prod(i,"lab_v"));
tax_prod(i,"lab_r") = lab_tax(i,"rate");
tax_prod("ave","lab_r") = lab_tax("ave","rate");
tax_prod("ave","cap_r") = cap_tax("ave","rate");
tax_prod("sd","lab_r") = lab_tax("sd","rate");
tax_prod("sd","cap_r") = cap_tax("sd","rate");

option tax_prod:2;
display tax_prod;
$batinclude gams2tbl tax_prod

*	Labor income tax
linc_tax("rate") = 100*tli0;
linc_tax("value_trl") = linctax;
option linc_tax:2
display linc_tax;

*	Asset income tax
cinc_tax("rate") = 100*tci0;
cinc_tax("value_trl") = cinctax;
option cinc_tax:2
display cinc_tax;

*	Consumption tax.
cons_tax("rate") = 100*tc;
cons_tax("value_bil") = sum(i, tc0(i) * (1+ts0(i)) * cd0(i));
option cons_tax:2
display cons_tax;

*	Summary for tax on household.
tax_house("lin","value") = linc_tax("value_trl");
tax_house("lin","rate") = linc_tax("rate");
tax_house("ain","value") = cinc_tax("value_trl");
tax_house("ain","rate") = cinc_tax("rate");
tax_house("cons","value") = round(cons_tax("value_bil"), 1);
tax_house("cons","rate") = cons_tax("rate");
option tax_house:1;
display tax_house;
$batinclude gams2tbl tax_house

set tax_comp / cap, lab, cons, linc, cinc, misc /;

*	Note: consumption subsidy is not considered here.

t_rev("cap","rev(trl)") = round(cap_tax("tot","value_bil")/1000, 1);
t_rev("lab","rev(trl)") = round(lab_tax("tot","value_bil")/1000, 1);
t_rev("cons","rev(trl)") = round(cons_tax("value_bil"), 1);
t_rev("linc","rev(trl)") = round(linc_tax("value_trl"), 1);
t_rev("cinc","rev(trl)") = round(cinc_tax("value_trl"), 1);
t_rev("misc","rev(trl)")
	= round(sum(i, tm0(i)*m0(i) + tq0(i)*(x0(i)+d0(i))), 1);
t_rev("tot","rev(trl)")
	= round(sum(tax_comp, t_rev(tax_comp,"rev(trl)")), 1);
t_rev(tax_comp,"share(\%)")
	= round(100 * t_rev(tax_comp,"rev(trl)")
	/ t_rev("tot","rev(trl)"), 1);
option t_rev:1;
display t_rev;
$batinclude gams2tbl t_rev
