$title	Dynamic model for double dividend analysis.
display "com: Dynamic model for double dividend analysis.";
$ontext
$Id: dd_bau_calibration.gms,v 1.1 2005/08/04 08:21:39 st Exp $
Copyright (C)	2004 Shiro Takeda
Time-stamp:	<2005-07-11 19:43:20 Shiro Takeda>
Author:		Shiro Takeda <zbc08106@park.zero.ad.jp>
e-mail:		<zbc08106@park.zero.ad.jp>
First-written:	<2003/04/24>
$offtext
*	------------------------------------------------------------------
*	Important parameters.

*	Scenario type
$if not setglobal scen $setglobal scen base

*	The number of periods.
$if not setglobal tyear $setglobal tyear 40

*	Emission reduction rate (%).
$if not setglobal rdrate $setglobal rdrate 100

*	Trade balance.  Non-zero means balanced trade at each period.
$if not setglobal bopcon $setglobal bopcon 0

*	Quadratic adjustment cost.
$if not setglobal phi $setglobal phi 0.5

*	Uncompensated elasticity of labor supply:
$if not setglobal eols $setglobal eols 0.19

*	Intertemporal elasticity:
$if not setglobal sigu $setglobal sigu 0.5

*	Coefficient for Armington elasticity:
$if not setglobal c_siga $setglobal c_siga 1

*	Coefficient for energy eos:
$if not setglobal c_sige $setglobal c_sige 1

*	Government budget balance.  Non-zero means balanced budget at each
*	period.
$if not setglobal gbb $setglobal gbb 0

*	Labor growth rate
$if not setglobal gr $setglobal gr 0

*	Solver choice for calibration.
$if not setglobal mcp $setglobal mcp 1

*	Solver choice
option mcp = path;
* option mcp = miles;

*	Digit.
option decimals = 8;

option sysout = on;

*	----------------------------------------------------------------------
*	Include model file:
display "com: Include model file:";

$include dd_model.gms

option solprint = off;
*	------------------------------------------------------------------
*	Benchmark replication with MCP
display "com: Benchmark replication with MCP";
dynamic_mcp.workspace = 300;
dynamic_mcp.iterlim = 0;
dynamic_mcp.optfile=1;
solve dynamic_mcp using mcp;

*	------------------------------------------------------------------
*	Cleanup calculation with MCP
display "com: Cleanup calculation with MCP";
dynamic_mcp.iterlim = 9000;
solve dynamic_mcp using mcp;
option solprint = on;

*	----------------------------------------------------------------------
*	Parameters for results:
parameter
    res_var
    chk_j
    chk_k
    chk_sl
    chk_cc
    chk_w

    chk_t_pf
    chk_t_ene

    chk_roc_t_pf
    chk_roc_t_ene

    chk_c_emi
    chk_v_gdp

    chk_q_g
;

chk_j(t,"ss") = q_j.l(t);
chk_k(t,"ss") = q_k.l(t);
chk_sl(t,"ss") = q_sl.l(t);
chk_cc(t,"ss") = q_cc.l(t);
chk_w(t,"ss") = q_w.l(t);
chk_c_emi(t,"ss") = c_emi.l(t);
chk_v_gdp(t,"ss") = v_gdp.l(t);
chk_q_g(t,"ss") = q_g.l(t);

*	------------------------------------------------------------------
*	BAU equilibrium calibration:
display "com: BAU equilibrium calibration:";

dcalib = 1;
display dcalib;

m_lum.fx = 0;

m_lum_p.l(t) = 0;
m_lum_p.lo(t) = -inf;
m_lum_p.up(t) = +inf;

*	Labor endowment:
$include roc_in_labor_force.gms
display roc_in_labor_force;

lref(tfirst) = endl;
loop(t, lref(t+1) = (1+roc_in_labor_force(t)) * lref(t));
display lref;

*	Technology parameter for primary factor:

t_pf.lo(t) = 0;
roc_t_pf.lo = -inf;

t_pf.up(t) = +inf;
roc_t_pf.up = +inf;

t_pf.l(t) = (1+0.012)**(ord(t) -1);
roc_t_pf.l = 0.012;

roc_gdp = 0.01304071;

*	Technology parameter for energy:

t_ene.lo(t) = 0;
roc_t_ene.lo = -inf;

t_ene.up(t) = +inf;
roc_t_ene.up = +inf;

t_ene.l(t) = (1+0.004)**(ord(t) -1);
roc_t_ene.l = 0.004;

roc_ca = 0.00322797;

*	Import initial values:
$if not %tyear%==40 $batinclude import_calibrated_bau.gms %tyear% %scen%
$if not %tyear%==40 $batinclude extend_initial_variables.gms %tyear%

solve dynamic_mcp using mcp;

*	Export results:
$batinclude export_calibrated_bau.gms %tyear% %scen%

res_var(t,"j") = q_j.l(t);
res_var(t,"k") = q_k.l(t);
res_var(t,"c") = q_cc.l(t);
res_var(t,"w") = q_w.l(t);
option res_var:3;
display res_var;

chk_j(t,"ns") = q_j.l(t);
chk_k(t,"ns") = q_k.l(t);
chk_sl(t,"ns") = q_sl.l(t);
chk_cc(t,"ns") = q_cc.l(t);
chk_w(t,"ns") = q_w.l(t);

chk_j(t,"ns/ss-1(%)") = 100 * (chk_j(t,"ns")/chk_j(t,"ss") - 1);
chk_k(t,"ns/ss-1(%)") =  100 * (chk_k(t,"ns")/chk_k(t,"ss") - 1);
chk_sl(t,"ns/ss-1(%)") = 100 * (chk_sl(t,"ns")/chk_sl(t,"ss") - 1);
chk_cc(t,"ns/ss-1(%)") = 100 * (chk_cc(t,"ns")/chk_cc(t,"ss") - 1);
chk_w(t,"ns/ss-1(%)") = 100 * (chk_w(t,"ns")/chk_w(t,"ss") - 1);
option chk_j:3, chk_k:3, chk_sl:3, chk_cc:3, chk_w:3;
display chk_j, chk_k, chk_sl, chk_cc, chk_w;

chk_t_pf(t) = t_pf.l(t);
chk_t_ene(t) = t_ene.l(t);
display chk_t_pf, chk_t_ene;

chk_roc_t_pf = roc_t_pf.l;
chk_roc_t_ene = roc_t_ene.l;
display chk_roc_t_pf, chk_roc_t_ene;

chk_c_emi(t,"ns") = c_emi.l(t);
chk_v_gdp(t,"ns") = v_gdp.l(t);
chk_c_emi(t,"roc(%)")$chk_c_emi(t-1,"ns")
    = 100 * (chk_c_emi(t,"ns")/chk_c_emi(t-1,"ns") - 1);
chk_v_gdp(t,"roc(%)")$chk_v_gdp(t-1,"ns")
    = 100 * (chk_v_gdp(t,"ns")/chk_v_gdp(t-1,"ns") - 1);
option chk_c_emi:2, chk_v_gdp:2;
display chk_c_emi, chk_v_gdp;

*	Post:
t_pf.fx(t) = t_pf.l(t);
t_ene.fx(t) = t_ene.l(t);

m_lum.l = 0;
m_lum.lo = -inf;
m_lum.up = +inf;
m_lum_p.fx(t) = m_lum_p.l(t);

chk_q_g(t,"ns") = q_g.l(t);
chk_q_g(t,"roc(%)")$q_g.l(t-1) = 100 * (q_g.l(t)/q_g.l(t-1) - 1);
option chk_q_g:3;
display chk_q_g;

$ontext
*	----------------------------------------------------------------------
*	Replicate check:
display "com: Replicate check:";

gqref(t) = g0 * q_g.l(t);
share_g.l(t) = p_disc.l(t) * g0 * p_g.l(t) * q_g.l(t)
    / sum(tt, p_disc.l(tt) * g0 * p_g.l(tt) * q_g.l(tt));
p_gle.l = sum(t, share_g.l(t) * p_disc.l(t) * p_g.l(t)) / pgle0;

* dynamic_mcp.iterlim = 0;
solve dynamic_mcp using mcp;
$offtext

* --------------------
* Local Variables:
* mode: gams
* fill-column: 78
* End:
