$title	Dynamic model for double dividend analysis.
display "com: Dynamic model for double dividend analysis.";
$ontext
$Id: dd_emi_reg.gms,v 1.1 2005/08/04 08:21:38 st Exp $
Copyright (C)	2004 Shiro Takeda
Time-stamp:	<2014-03-24 11:21:50 st>
Author:		Shiro Takeda <zbc08106@park.zero.ad.jp>
e-mail:		<zbc08106@park.zero.ad.jp>
First-written:	<2003/04/24>

$offtext
*	------------------------------------------------------------------
*	Important parameters.

*	Swapped tax
$if not setglobal s_tax $setglobal s_tax lum

*	Scenario type
$if not setglobal scen $setglobal scen base

*	The number of periods.
$if not setglobal tyear $setglobal tyear 100

*	Emission reduction rate (%).
$if not setglobal rdrate $setglobal rdrate 100

*	Trade balance.  Non-zero means balanced trade at each period.
$if not setglobal bopcon $setglobal bopcon 0

*	Quadratic adjustment cost.
$if not setglobal phi $setglobal phi 0.5

*	Uncompensated elasticity of labor supply:
$if not setglobal eols $setglobal eols 0.19

*	Intertemporal elasticity:
$if not setglobal sigu $setglobal sigu 0.5

*	Coefficient for Armington elasticity:
$if not setglobal c_siga $setglobal c_siga 1

*	Coefficient for energy eos:
$if not setglobal c_sige $setglobal c_sige 1

*	Government budget balance.  Non-zero means balanced budget at each
*	period.
$if not setglobal gbb $setglobal gbb 0

*	Labor growth rate
$if not setglobal gr $setglobal gr 0

*	Solver choice for calibration.
$if not setglobal mcp $setglobal mcp 1

*	Solver choice
option mcp = path;
* option mcp = miles;

*	End-of-line comment symbol.
$eolcom !
*	Inline comment symbol.
$inlinecom { }

*	Digit.
option decimals = 8;

option sysout = on;

*	----------------------------------------------------------------------
*	Include model file:
display "com: Include model file:";

$include dd_model.gms

$ontext
*	------------------------------------------------------------------
*	Benchmark replication with MCP
display "com: Benchmark replication with MCP";
dynamic_mcp.workspace = 300;
dynamic_mcp.iterlim = 0;
dynamic_mcp.optfile=1;
solve dynamic_mcp using mcp;

*	------------------------------------------------------------------
*	Cleanup calculation with MCP
display "com: Cleanup calculation with MCP";
dynamic_mcp.iterlim = 9000;
solve dynamic_mcp using mcp;
$offtext

$batinclude import_calibrated_bau.gms ok %scen%

*	----------------------------------------------------------------------
*	Replication check:
display "com: Replication check:";

t_pf.fx(t) = t_pf.l(t);
t_ene.fx(t) = t_ene.l(t);

tins("lum") = yes;
m_lum.l = 0;
m_lum.lo = -inf;
m_lum.up = +inf;
m_lum_p.fx(t) = m_lum_p.l(t);

*	Labor endowment path:
lref(t) = lref_100(t);
display lref;

*	Path of government expenditure:
gqref(t) = gqref_100(t);
display gqref;

*	Replication check:
dynamic_mcp.iterlim = 0;
solve dynamic_mcp using mcp;
dynamic_mcp.iterlim = 9000;

*	----------------------------------------------------------------------
*	Calculate utility change from carbon restriction.
display "com: Calculate utility change from carbon restriction.";

$include dd_emi_reg_par.gms

*	Swapped tax:
tins(taxinst) = no;
tins("%s_tax%") = yes;
display tins;
$include tax_scenario_emi_reg.gms

*	Reduction rates.
set	sce_rd		Reduction scenario
	/ 25,50,75,100 /;
parameter rd_rate	Reduction rate;
rd_rate("25") = 25;
rd_rate("50") = 50;
rd_rate("75") = 75;
rd_rate("100") = 100;
display rd_rate;

loop(sce_rd,

*	Reduction rate.
    rdrate = rd_rate(sce_rd);
    display rdrate;

*	Carbon restriction.
    carbl(t) =
	c_emi_100(t)
	- (rdrate/100)
	* (c_emi_100(t) - c_emi_100("1"));
    display carbl;

    option solprint = off;
    solve dynamic_mcp using mcp;
    option solprint = on;
    chk_numeraire = v_inc_h.m;
    display chk_numeraire;

$include check_multiplier.gms

    pc_util(sce_rd,"%s_tax%") = 100*(q_u.l - q_u_100);

*	Important variables.
    lev_k(t,sce_rd) = q_k.l(t);
    lev_j(t,sce_rd) = q_j.l(t);
    lev_sl(t,sce_rd) = q_sl.l(t);
    lev_cc(t,sce_rd) = q_cc.l(t);
    lev_w(t,sce_rd) = q_w.l(t);
    lev_emi(t,sce_rd) = c_emi.l(t);

    pc_k(t,sce_rd) = 100 * (q_k.l(t)/q_k_100(t) - 1);
    pc_j(t,sce_rd) = 100 * (q_j.l(t)/q_j_100(t) - 1);
    pc_sl(t,sce_rd) = 100 * (q_sl.l(t)/q_sl_100(t) - 1);
    pc_cc(t,sce_rd) = 100 * (q_cc.l(t)/q_cc_100(t) - 1);
    pc_w(t,sce_rd) = 100 * (q_w.l(t)/q_w_100(t) - 1);
    pc_emi(t,sce_rd) = 100 * (c_emi.l(t)/c_emi_100(t) - 1);

);

display lev_k, lev_j, lev_sl, lev_cc, lev_w;
display pc_k, pc_j, pc_sl, pc_cc, pc_w;
option chk_tax_multiplier:3;
display chk_tax_multiplier;
option pc_util:5;
display pc_util;

*	----------------------------------------------------------------------
*	Export results:
file unit / result_%s_tax%.txt /;
put unit;
$batinclude gams2txt pc_util


* --------------------
* Local Variables:
* mode: gams
* fill-column: 78
* End:
