$if not setglobal debug $offlisting offinclude

$hidden  gams2tbl.gms    Write a parameter into a table.
$hidden
$hidden  $...include gams2tbl item
$hidden
$hidden  Works for two-dimensional parameters and variables.
$hidden
$hidden  First invocation must be outside of a loop or if block.
$hidden
$hidden  Use a blank invocation (without argument) to initialize.
$hidden

*       Skip compilation is there is a pre-existing program error:

$if not errorfree $exit

*       If we are already initialized, skip declarations:

$if setglobal gams2tbl $goto start

*       Install an environment variable indicating that the
*       gams2tbl subsystem is in operation:

$setglobal gams2tbl='yes'

*       Declare temporary variables used for data transfer.

alias(tblu_1,tblu_2,*);

set     tblr    Ordered set for table rows      /tblr1*tblr1000/, 
        tblc    Ordered set for table columns   /tblc1*tblc1000/;

set tblrow(*), tblcol(*);
alias (tblrow,tblrows), (tblcol,tblcols);

parameter tbl_rv(*), tbl_cv(*);  
scalar tblcount, tbllwidth, tbltw, tbllw, tbltj, tbltf, tblnd, tblnw;

$if errorfree $goto start

$onlisting

*       An error has occured during the initial invocation of
*       gams2tbl. There are two reasons than an error may have
*       been generated here:

*       (i) The first invocation of gams2tbl is inside an IF block or LOOP.  
*
*       The solution for this problem is to use initialize with 
*       "$libinclude gams2tbl" before entering the LOOP or IF.

*       (ii) User program has used one of the gams2tbl reserved names:
*       tblrow, tblcol, tblrows, tblcols, tbl_rv, tbl_cv, tblcount,
*       tbllwidth, tbltw, tbltj, tbltf, tblnd, tblnw, tbllw, tblu_1 or tblu_2.

$abort 'Error on initial $libinclude gams2tbl.'

$label start

*       Blank invocation -- nothing to do:

$if %1a==a $exit

*       See that the item is declared and defined:

$if declared %1    $goto declared
$error Error in gams2tbl: identfier %1 is undeclared.
$exit

$label declared
$if defined %1     $goto defined
$error Error in gams2tbl: identfier %1 is undefined.
$exit

$label defined

*       We may not want to limit our attention to two-dimensional
*       items at some point:

$if dimension 2 %1  $goto array
$error Error in gams2tbl: item %1 is not two-dimensional.

$label array

*       The following statement permits references into the 
*       universal set:
$onuni

*       Save current file status to restore the state:

tbltw = file.tw;
tbllw = file.lw;
tbltj = file.tj;
tbltf = file.tf;
tblnd = file.nd;
tblnw = file.nw;

$if not setglobal zeros $setglobal zeros yes
$if not setglobal endtable $setglobal  endtable yes

$if not setglobal row_set tblrow(tblu_1)    = yes$sum(tblu_2$%1(tblu_1,tblu_2), 1);
$if not setglobal col_set tblcol(tblu_2)    = yes$sum(tblu_1$%1(tblu_1,tblu_2), 1);
$if     setglobal row_set tblrow(%row_set%) = yes;
$if     setglobal col_set tblcol(%col_set%) = yes;

$if not setglobal row_label $setlocal rl tblrow
$if     setglobal row_label $setlocal rl %row_label%
$if not setglobal col_label $setlocal cl tblcol
$if     setglobal col_label $setlocal cl %col_label%

*  Determine the maximum length row descriptor:

file.lw = 0;  tbllwidth = 0;
loop(tblrow,  put @1, %rl%.te(tblrow); tbllwidth = max(tbllwidth, file.cc-1););
file.tw = tbllwidth;  put @1,' '/;

*  But permit the user to over-ride the label width:

$if setglobal labelwidth tbllwidth=%labelwidth%;

$if     settype %1 $set data "%1.te"
$if not settype %1 $set data "%1"

*       Assign the default row and column orders based on the 
*       GAMS symbol table:

$label count
tblcount = 0; loop(tblcol, tblcount = tblcount + 1;  tbl_cv(tblcol)=tblcount;);
tblcount = 0; loop(tblrow, tblcount = tblcount + 1;  tbl_rv(tblrow)=tblcount;);

*       Use the user-specified sort order if provided:

$if not setglobal row_order $goto colsort
$if "%row_order%"=="" $goto colsort
loop(tblrow, tbl_rv(tblrow) 
    = 1 + sum(tblrows$(%row_order%(tblrows) lt %row_order%(tblrow)), 1););

*       Resolve ties using the GAMS symbol table ordering:

loop(tblr$(ord(tblr) le card(tblrow)),
  tblcount = 0;
  loop(tblrow$(tbl_rv(tblrow) eq  ord(tblr)),
    tbl_rv(tblrow) = tbl_rv(tblrow) + tblcount;
    tblcount = tblcount + 1;
  );
);


$label colsort
$if not setglobal col_order $goto chkformat
$if "%col_order%"=="" $goto chkformat
loop(tblcol, tbl_cv(tblcol) 
    = 1 + sum(tblcols$(%col_order%(tblcols) lt %col_order%(tblcol)), 1););

*       Resolve ties using the GAMS symbol table ordering:

loop(tblc$(ord(tblc) le card(tblcol)),
  tblcount = 0;
  loop(tblcol$(tbl_cv(tblcol) eq  ord(tblc)),
    tbl_cv(tblcol) = tbl_cv(tblcol) + tblcount;
    tblcount = tblcount + 1;
  );
);


*       Branch to user-specified format:

$label chkformat
$if not setglobal format $goto txt
$if %format%==txt  $goto txt
$if %format%==TXT  $goto txt
$if %format%==gms  $goto gms
$if %format%==GMS  $goto gms
$if %format%==prn  $goto prn
$if %format%==PRN  $goto prn
$if %format%==tex  $goto tex
$if %format%==TEX  $goto tex
$if %format%==html $goto html
$if %format%==HTML $goto html
$abort Format must be txt, gms, prn, tex, or html for gams2tbl: %format%

*       Generate a text format:

$label txt

file.tw = tbllwidth+card(tblcol)*file.nw;
file.tj = 2;
$if not setglobal title_just $goto writetitle
$if %title_just%==right  file.tj=1;
$if %title_just%==center file.tj=3;

$label writetitle
$if not setglobal title_just file.tw=0;
file.lw=0;
$if     setglobal title put '%title%'/; 
$if not setglobal title put %1.ts/; 
$if "%title%"=="" put %1.ts/; 

file.tw=1;
loop(tblc$(ord(tblc) le tbllwidth+card(tblcol)*file.nw),put '-';); put /;
put @tbllwidth' ';
file.tw = file.nw;  file.tj = 1;
loop(tblc$(ord(tblc) le card(tblcol)),
  loop(tblcol$(tbl_cv(tblcol) eq ord(tblc)), put %cl%.te(tblcol);  ));
put /;
file.tw=1;
loop(tblc$(ord(tblc) le tbllwidth+card(tblcol)*file.nw),put '-';); put /;

loop(tblr$(ord(tblr) le card(tblrow)),
  loop(tblrow$(tbl_rv(tblrow) eq ord(tblr)),
    file.tw = tbllwidth; file.tj = 2;
    put %rl%.te(tblrow);
    file.lw = file.nw;
    loop(tblc$(ord(tblc) le card(tblcol)),
      loop(tblcol$(tbl_cv(tblcol) eq ord(tblc)),
$if setglobal r_decimals file.nd = %r_decimals%(tblrow);
$if setglobal c_decimals file.nd = %c_decimals%(tblcol);

$if settype %1 file.tf=0; file.tw=file.nw; file.tj=1;
$if     "%zeros%"=="yes"  put %data%(tblrow,tblcol);
$if not "%zeros%"=="yes"  if (%1(tblrow,tblcol), put %data%(tblrow,tblcol); else file.tw = file.nw; put '%zeros%'; );
$if settype %1 file.tf=tbltf; file.tj=2;

    ));
  ); put /; 
);

file.tw=1;
loop(tblc$(ord(tblc) le tbllwidth+card(tblcol)*file.nw),put '-';); put //;
$goto cleanup

$label gms

file.tw = tbllwidth+card(tblcol)*file.nw;
file.tj = 2;
$if not setglobal title_just $goto writegmstitle
$if %title_just%==right  file.tj=1;
$if %title_just%==center file.tj=3;

$label writegmstitle
$if not setglobal title_just file.tw=0;
file.lw=0;
$if     setglobal title put 'table %1 "%title%"'/; 
$if not setglobal title put 'table %1 "'%1.ts'"'/; 
$if "%title%"=="" put 'table %1 "'%1.ts'"'/; 

file.tw=1;
*.loop(tblc$(ord(tblc) le tbllwidth+card(tblcol)*file.nw),put '-';); put /;
put @tbllwidth' ';
file.tw = file.nw;  file.tj = 1;
loop(tblc$(ord(tblc) le card(tblcol)),
  loop(tblcol$(tbl_cv(tblcol) eq ord(tblc)), put %cl%.te(tblcol);  ));
put /;
file.tw=1;
*.loop(tblc$(ord(tblc) le tbllwidth+card(tblcol)*file.nw),put '-';); put /;

loop(tblr$(ord(tblr) le card(tblrow)),
  loop(tblrow$(tbl_rv(tblrow) eq ord(tblr)),
    file.tw = tbllwidth; file.tj = 2;
    put %rl%.te(tblrow);
    file.lw = file.nw;
    loop(tblc$(ord(tblc) le card(tblcol)),
      loop(tblcol$(tbl_cv(tblcol) eq ord(tblc)),
$if setglobal r_decimals file.nd = %r_decimals%(tblrow);
$if setglobal c_decimals file.nd = %c_decimals%(tblcol);

$if settype %1 file.tf=0; file.tw=file.nw; file.tj=1;
$if     "%zeros%"=="yes"  put %data%(tblrow,tblcol);
$if not "%zeros%"=="yes"  if (%1(tblrow,tblcol), put %data%(tblrow,tblcol); else file.tw = file.nw; put '%zeros%'; );
$if settype %1 file.tf=tbltf; file.tj=2;

    ));
  ); put /; 
);

file.tw=1;
*loop(tblc$(ord(tblc) le tbllwidth+card(tblcol)*file.nw),put '-';); put /;
put /;
$goto cleanup



$label prn
file.lw=0;  file.tw=0;
        put     '"';
$if     setglobal title put '%title%'; 
$if not setglobal title put %1.ts; 
$if "%title%"=="" put %1.ts/; 
        put '"'//;
        put '" " ';
        loop(tblc$(ord(tblc) le card(tblcol)),
          loop(tblcol$(tbl_cv(tblcol) eq ord(tblc)),
            put '"',%cl%.te(tblcol),'" ';));  
        put /;
        loop(tblr$(ord(tblr) le card(tblrow)),
          loop(tblrow$(tbl_rv(tblrow) eq ord(tblr)),
            put '"'%rl%.te(tblrow)'" ';

            tblcount = 0;
            loop(tblc$(ord(tblc) le card(tblcol)),
$if not settype %1 tblcount = tblcount + sum(tblcol$(tbl_cv(tblcol) eq ord(tblc)), 1$%data%(tblrow,tblcol));
            );
$if settype %1 tblcount = 1         
            if (tblcount,
              loop(tblc$(ord(tblc) le card(tblcol)),
                loop(tblcol$(tbl_cv(tblcol) eq ord(tblc)),
$if setglobal r_decimals file.nd = %r_decimals%(tblrow);
$if setglobal c_decimals file.nd = %c_decimals%(tblcol);
$if settype %1 file.tf=0; put '"';
                  put %data%(tblrow,tblcol);
$if settype %1 file.tf=tbltf; put '"';
            )));
          ) put /; ); 
          put //;
$goto cleanup

$label tex
file.lw=0;  file.tw=0; file.nw=0; 
put     '\begin{table}[ht]' /
        '\caption{';
$if     setglobal title put '%title%'; 
$if not setglobal title put %1.ts; 
$if "%title%"=="" put %1.ts/; 
put     '}'    /
        '\begin{center}'  /
        '\begin{tabular}{l';
        loop(tblcol, put 'r';) put '}  \hline '  /;
        loop(tblc$(ord(tblc) le card(tblcol)),
          loop(tblcol$(tbl_cv(tblcol) eq ord(tblc)),
            put ' &\multicolumn{1}{c}{',%cl%.te(tblcol),'}'/;)); 
        put ' \\ \hline ' /;
        loop(tblr$(ord(tblr) le card(tblrow)),
          loop(tblrow$(tbl_rv(tblrow) eq ord(tblr)),
          put %rl%.te(tblrow);
          loop(tblc$(ord(tblc) le card(tblcol)),
            loop(tblcol$(tbl_cv(tblcol) eq ord(tblc)),
$if setglobal r_decimals file.nd = %r_decimals%(tblrow);
$if setglobal c_decimals file.nd = %c_decimals%(tblcol);
              put ' & ';
$if settype %1 file.tf=0; 
$if     "%zeros%"=="yes"  put %data%(tblrow,tblcol);
$if not "%zeros%"=="yes"  if (%1(tblrow,tblcol), put %data%(tblrow,tblcol) else put '%zeros%'; );
$if settype %1 file.tf=tbltf; 
          )); put '  \\'/;
        )); put '\hline' /;
put     '\end{tabular}' /'\end{center}'/;
$if %endtable%==yes put '\end{table}'//;
$if not %endtable%==yes put //;
$goto cleanup

$label html

        file.lw=0;  file.tw=0; file.nw=0; 
        put '<table border=0 width="50%"><caption>'/;
file.lw=0;
$if     setglobal title put '%title%'/; 
$if not setglobal title put %1.ts/; 
$if "%title%"=="" put %1.ts/; 
        put '</caption><tbody><tr><td> </td>'/;
        loop(tblc$(ord(tblc) le card(tblcol)),
          loop(tblcol$(tbl_cv(tblcol) eq ord(tblc)),
            put @10'<th align="right">',%cl%.te(tblcol),'</th>'/;));  
        put '</tr>'/;
        put '<tr><td colspan='; 
        file.nd=0; put (1+card(tblcol)); file.nd=tblnd;
        put '><hr></td></tr>'/;
        loop(tblr$(ord(tblr) le card(tblrow)),
          loop(tblrow$(tbl_rv(tblrow) eq ord(tblr)),
            put '<tr><th align="left">',%rl%.te(tblrow),'</th>'/;
            loop(tblc$(ord(tblc) le card(tblcol)),
              loop(tblcol$(tbl_cv(tblcol) eq ord(tblc)),
$if setglobal r_decimals file.nd = %r_decimals%(tblrow);
$if setglobal c_decimals file.nd = %c_decimals%(tblcol);
                put @10'<td align="right">';
$if settype %1 file.tf=0; 
$if     "%zeros%"=="yes"  put %data%(tblrow,tblcol);
$if not "%zeros%"=="yes"  if (%1(tblrow,tblcol), put %data%(tblrow,tblcol) else file.tw = file.nw; put '%zeros%'; );
$if settype %1 file.tf=tbltf; 
                put '</td>'/;
            )); 
          );
          put '</tr>'/; 
        ); 
        put '<tr><td colspan='; 
        file.nd=0; put (1+card(tblcol)); file.nd=tblnd;
        put '><hr></td></tr></table>'//;

$label cleanup
$offuni
option clear=tblrow;
option clear=tblcol;
option clear=tbl_rv;
option clear=tbl_cv;


*       Restore the put status:

file.tw = tbltw;
file.lw = tbllw;
file.tj = tbltj;
file.nd = tblnd;
file.nw = tblnw;


